@@ -34,15 +34,15 @@
 #define _WIN32_WINNT 0x0501
 
 #define PACKAGE "BitlBee"
-#define BITLBEE_VERSION "1.2.8"
+#define BITLBEE_VERSION "1.3dev"
 #define VERSION BITLBEE_VERSION
 #define BITLBEE_VER(a,b,c) (((a) << 16) + ((b) << 8) + (c))
-#define BITLBEE_VERSION_CODE BITLBEE_VER(1, 2, 8)
+#define BITLBEE_VERSION_CODE BITLBEE_VER(1, 3, 0)
 
 #define MAX_STRING 511
 
 #if HAVE_CONFIG_H
-#include "config.h"
+#include <config.h>
 #endif
 
 #include <fcntl.h>
@@ -125,6 +125,7 @@
 #define HELP_FILE VARDIR "help.txt"
 #define CONF_FILE_DEF ETCDIR "bitlbee.conf"
 
+#include "bee.h"
 #include "irc.h"
 #include "storage.h"
 #include "set.h"
@@ -159,8 +160,9 @@ int bitlbee_inetd_init( void );
 gboolean bitlbee_io_current_client_read( gpointer data, gint source, b_input_condition cond );
 gboolean bitlbee_io_current_client_write( gpointer data, gint source, b_input_condition cond );
 
-void root_command_string( irc_t *irc, user_t *u, char *command, int flags );
+void root_command_string( irc_t *irc, char *command );
 void root_command( irc_t *irc, char *command[] );
+gboolean cmd_identify_finish( gpointer data, gint fd, b_input_condition cond );
 gboolean bitlbee_shutdown( gpointer data, gint fd, b_input_condition cond );
 
 char *set_eval_root_nick( set_t *set, char *new_nick );
