@@ -304,14 +304,18 @@ void http_encode( char *s )
 	
 	for( i = j = 0; t[i]; i ++, j ++ )
 	{
-		if( !isalnum( t[i] ) && !strchr( "._-~", t[i] ) )
+		/* Warning: isalnum() is locale-aware, so don't use it here! */
+		if( ( t[i] >= 'A' && t[i] <= 'Z' ) ||
+		    ( t[i] >= 'a' && t[i] <= 'z' ) ||
+		    ( t[i] >= '0' && t[i] <= '9' ) ||
+		    strchr( "._-~", t[i] ) )
 		{
-			sprintf( s + j, "%%%02X", ((unsigned char*)t)[i] );
-			j += 2;
+			s[j] = t[i];
 		}
 		else
 		{
-			s[j] = t[i];
+			sprintf( s + j, "%%%02X", ((unsigned char*)t)[i] );
+			j += 2;
 		}
 	}
 	s[j] = 0;
@@ -646,3 +650,51 @@ int md5_verify_password( char *password, char *hash )
 
 	return ret;
 }
+
+char **split_command_parts( char *command )
+{
+	static char *cmd[IRC_MAX_ARGS+1];
+	char *s, q = 0;
+	int k;
+	
+	memset( cmd, 0, sizeof( cmd ) );
+	cmd[0] = command;
+	k = 1;
+	for( s = command; *s && k < IRC_MAX_ARGS; s ++ )
+		if( *s == ' ' && !q )
+		{
+			*s = 0;
+			while( *++s == ' ' );
+			if( *s == '"' || *s == '\'' )
+			{
+				q = *s;
+				s ++;
+			}
+			if( *s )
+			{
+				cmd[k++] = s;
+				s --;
+			}
+			else
+			{
+				break;
+			}
+		}
+		else if( *s == '\\' && ( ( !q && s[1] ) || ( q && q == s[1] ) ) )
+		{
+			char *cpy;
+			
+			for( cpy = s; *cpy; cpy ++ )
+				cpy[0] = cpy[1];
+		}
+		else if( *s == q )
+		{
+			q = *s = 0;
+		}
+	
+	/* Full zero-padding for easier argc checking. */
+	while( k <= IRC_MAX_ARGS )
+		cmd[k++] = NULL;
+	
+	return cmd;
+}
