@@ -35,6 +35,14 @@
 #include <ctype.h>
 #include <errno.h>
 
+/* GLib < 2.12.0 doesn't have g_ascii_strtoll(), work around using system strtoll(). */
+/* GLib < 2.12.4 can be buggy: http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=488013 */
+#if !GLIB_CHECK_VERSION(2,12,5)
+#include <stdlib.h>
+#include <limits.h>
+#define g_ascii_strtoll strtoll
+#endif
+
 #define TXL_STATUS 1
 #define TXL_USER 2
 #define TXL_ID 3
@@ -65,6 +73,8 @@ static void twitter_groupchat_init(struct im_connection *ic);
  */
 static void txu_free(struct twitter_xml_user *txu)
 {
+	if (txu == NULL)
+		return;
 	g_free(txu->name);
 	g_free(txu->screen_name);
 	g_free(txu);
@@ -88,6 +98,8 @@ static void txs_free(struct twitter_xml_status *txs)
 static void txl_free(struct twitter_xml_list *txl)
 {
 	GSList *l;
+	if (txl == NULL)
+		return;
 	for ( l = txl->list; l ; l = g_slist_next(l) )
 		if (txl->type == TXL_STATUS)
 			txs_free((struct twitter_xml_status *)l->data);
@@ -104,7 +116,7 @@ static void twitter_add_buddy(struct im_connection *ic, char *name, const char *
 	struct twitter_data *td = ic->proto_data;
 
 	// Check if the buddy is allready in the buddy list.
-	if (!imcb_find_buddy( ic, name ))
+	if (!bee_user_by_handle( ic->bee, ic, name ))
 	{
 		char *mode = set_getstr(&ic->acc->set, "mode");
 		
@@ -112,7 +124,12 @@ static void twitter_add_buddy(struct im_connection *ic, char *name, const char *
 		imcb_add_buddy( ic, name, NULL );
 		imcb_rename_buddy( ic, name, fullname );
 		if (g_strcasecmp(mode, "chat") == 0)
+		{
+			/* Necessary so that nicks always get translated to the
+			   exact Twitter username. */
+			imcb_buddy_nick_hint( ic, name, name );
 			imcb_chat_add_buddy( td->home_timeline_gc, name );
+		}
 		else if (g_strcasecmp(mode, "many") == 0)
 			imcb_buddy_status( ic, name, OPT_LOGGED_IN, NULL, NULL );
 	}
@@ -446,7 +463,7 @@ static void twitter_groupchat_init(struct im_connection *ic)
 	
 	td->home_timeline_gc = gc = imcb_chat_new( ic, "home/timeline" );
 	
-	name_hint = g_strdup_printf( "Twitter_%s", ic->acc->user );
+	name_hint = g_strdup_printf( "%s_%s", td->prefix, ic->acc->user );
 	imcb_chat_name_hint( gc, name_hint );
 	g_free( name_hint );
 }
@@ -472,6 +489,9 @@ static void twitter_groupchat(struct im_connection *ic, GSList *list)
 	for ( l = list; l ; l = g_slist_next(l) )
 	{
 		status = l->data;
+		if (status->user == NULL || status->text == NULL)
+			continue;
+
 		twitter_add_buddy(ic, status->user->screen_name, status->user->name);
 		
 		strip_html(status->text);
@@ -503,7 +523,7 @@ static void twitter_private_message_chat(struct im_connection *ic, GSList *list)
 
 	if( mode_one )
 	{
-		g_snprintf( from, sizeof( from ) - 1, "twitter_%s", ic->acc->user );
+		g_snprintf( from, sizeof( from ) - 1, "%s_%s", td->prefix, ic->acc->user );
 		from[MAX_STRING-1] = '\0';
 	}
 	
@@ -735,4 +755,4 @@ void twitter_friendships_create_destroy(struct im_connection *ic, char *who, int
 	args[0] = "screen_name";
 	args[1] = who;
 	twitter_http(ic, create ? TWITTER_FRIENDSHIPS_CREATE_URL : TWITTER_FRIENDSHIPS_DESTROY_URL, twitter_http_post, ic, 1, args, 2);
-}
\ No newline at end of file
+}
